#include <sta/tacos/system/events.hpp>
#include <sta/rtos/event.hpp>

#include <sta/debug/assert.hpp>

#include <cmsis_os2.h>
#include <FreeRTOS.h>


namespace
{
	// Event handle
	sta::RtosEvent * systemEvents = nullptr;
}


namespace sta
{
	namespace tacos
	{
		void initSystemEvents()
		{
			if (systemEvents == nullptr)
			{
				systemEvents = new sta::RtosEvent();
			}
		}

		void signalSystemEvents(uint32_t flags)
		{
			STA_ASSERT_MSG(systemEvents != nullptr, "System events not initialized");
			systemEvents->set(flags);
		}

		void waitForSystemEvents(uint32_t flags, uint32_t options, uint32_t timeout)
		{
			STA_ASSERT_MSG(systemEvents != nullptr, "System events not initialized");
			systemEvents->peek(flags, timeout);
		}

		void signalStartupEvent()
		{
			signalSystemEvents(STA_TACOS_SYSTEM_EVENTS_STARTUP);
		}
	
		void waitForStartupEvent()
		{
			waitForSystemEvents(STA_TACOS_SYSTEM_EVENTS_STARTUP, osFlagsWaitAll, osWaitForever);
		}
	} // namespace tacos
} // namespace sta
